/*
** Module   : AstManagerPlugin.java
** Abstract : defines the API for an AstManager plugin that implements services
**            for creating, deleting, editing and using ASTs
**
** Copyright (c) 2009-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -----------------------------Description-----------------------------
** 001 GES 20090430   @42199 Created initial version. This defines the API for
**                           an AstManager plugin that implements provides
**                           services for creating, deleting, editing and using
**                           ASTs.
** 002 GES 20090518   @42369 Import change.
** 003 CA  20131023          Added isExistingAst API, to check if an AST exists in the plugin's
**                           storage. 
*/

package com.goldencode.ast;

/**
 * This defines the API for an {@link AstManager} plugin that implements
 * services for creating, deleting, editing and using Abstract Syntax Trees
 * (ASTs). The use of the term AST (by itself) refers to the entire tree. To
 * refer to a single node of the tree, the term "AST node" or "node" is used.
 * <p>
 * At the core of these services is the idea that a database or registry of
 * trees exist.
 * <p>
 * Maintains a project-specific bidirectional indexing facility for mapping
 * ASTs to project-unique IDs.  Given a 64-bit unique ID, one can lookup the
 * specific AST name that is associated with that ID. Likewise, given an AST
 * name, one can obtain the 64-bit unique ID.  In addition, this class tracks
 * the next available 64-bit node ID for each tree.
 * <p>
 * A <code>long</code> is used to store and represent the 64-bit IDs.  These 
 * 64-bit IDs are 'partitioned' into an upper DWORD (32-bit unsigned value) and
 * a lower DWORD. The upper DWORD is associated with the name that is the source
 * of a given AST.  Each tree name in a project will have a unique upper DWORD
 * that is assigned when the tree is added to the registry using the
 * {@link #addTree} method.  All AST nodes in the tree created from/for that
 * tree will share the same upper DWORD.  This upper DWORD is called the
 * 'tree portion' of the ID.  A 'tree ID' is an ID with the lower DWORD set
 * to 0x00000000 and the upper DWORD set to the tree portion of the specific
 * tree being referenced.  The lower DWORD holds a value that is unique to
 * each AST node in that tree.  This lower DWORD is called the 'node portion'
 * of the ID.  When a node portion is combined with the tree portion of an
 * ID, the result is a 64-bit ID that uniquely identifies a specific AST node
 * in a specific tree.  For this reason, such IDs can be used as persistent 
 * references between ASTs in different trees or even between AST nodes in
 * the same tree.
 * <p>
 * An ID of 0xFFFFFFFFFFFFFFFF is invalid and is the equivalent to a
 * <code>null</code> ID.  Similarly, a tree ID (which by definition has a
 * a lower DWORD of 0x00000000) can never refer to a specific node and only
 * refers to a tree (to the entire tree rather than any given node of that
 * tree).
 * <p>
 * This class provides the assignment facility for obtaining unique tree IDs
 * as well as unique node IDs (see {@link #getNextNodeId}).
 * <p>
 * <b>Name lookups are done on a case sensitive basis.</b>
 * <p>
 * To facilitate multi-threaded applications, a tree level locking facility
 * is provided. {@link #lockTree} and {@link #unlockTree} provide a mechanism
 * to obtain and release this lock respectively. This locking has nothing to
 * do with file system locks nor is there any non-Java locking backing this
 * lock. Rather, this lock is a cooperative agreement between all callers
 * that no use of the given tree will be made except when the caller holds
 * this exclusive lock to the tree.
 */
public interface AstManagerPlugin
{
   /**
    * Lookup the name associated with the tree specified by this ID. The ID can
    * be a tree ID (an ID that represents an entire AST) or it can be the ID
    * an individual AST node ID. Node IDs encode the tree ID and a portion that
    * is node-specific. The node-specific portion will be ignored.
    *
    * @param    id
    *           The ID to use as a key in the lookup. This ID may be a tree ID
    *           or a specific node ID.
    *
    * @return   The name associated with the tree or <code>null</code> if no
    *           such tree ID exists.
    */
   public String getTreeName(long id);
   
   /**
    * Reverse lookup of the ID that corresponds to a given AST name.
    *
    * @param    name
    *           Tree name whose unique ID should be returned.
    *
    * @return   The tree ID associated with the name or
    *           {@link AstManager#INVALID_ID} if the tree name doesn't exist or
    *           if it is <code>null</code>.
    */
   public long getTreeId(String name);
   
   /**
    * Returns the tree ID portion of a node ID.
    *
    * @return   The ID which corresponds to the tree itself, with all
    *           node ID portions removed (masked off) or
    *           {@link AstManager#INVALID_ID} if the ID is invalid.
    */
   public long getTreeId(long id);
   
   /**
    * Load a named AST from persistent storage.
    *
    * @param    name
    *           The name of the tree. Often this is a file name.
    *
    * @return   The loaded AST.
    *
    * @throws   AstException
    *           On any error during the loading process.
    * @throws   IllegalArgumentException
    *           If the name is invalid or the specified resource doesn't exist.
    */
   public Aast loadTree(String name)
   throws AstException,
          IllegalArgumentException;
   
   /**
    * Save the given AST to persistent storage under the given name.
    *
    * @param    ast
    *           The tree to save.
    * @param    name
    *           The name of the tree. Often this is a file name.
    * @param    terse
    *           Do not store line and column data when the node data is saved.
    * 
    * @throws   AstException
    *           On any error during the saving process.
    * @throws   IllegalArgumentException
    *           If the name is invalid or the specified resource cannot be
    *           used as a target to store the AST.
    */
   public void saveTree(Aast ast, String name, boolean terse)
   throws AstException,
          IllegalArgumentException;
   
   /**
    * Obtains a process-wide lock on any processing related to the given tree.
    * If the lock is already held, this method will block until it obtains
    * the lock. When this method returns normally, the calling thread is
    * guaranteed to be holding the lock. This may be called recursively by the
    * same thread and if the current thread is already holding the lock, then
    * this method will only update the reference count for the lock but
    * otherwise it will immediately return. As a consequence, the lock will not
    * be released unless {@link #unlockTree} is called the same number of times
    * as this lock method. Thus the calls to this lock method and to
    * <code>unlock</code> must always be balanced.
    * <p>
    * This locking has nothing to do with file system locks nor is there any
    * non-Java locking backing this lock. Rather, this lock is a cooperative
    * agreement between all callers (within the same JVM) that no use of the
    * given tree will be made except when the caller holds this exclusive lock
    * to the resource.
    *
    * @param    id
    *           Tree ID that is associated with the lock.
    *
    * @throws   InterruptedException
    *           When an interruption occurs during blocking for the lock. If
    *           this is thrown, the caller will NOT be holding the lock.
    * @throws   IllegalArgumentException
    *           If the ID is not a valid tree ID.
    */
   public void lockTree(long id)
   throws InterruptedException,
          IllegalArgumentException;
   
   /**
    * Releases the process-wide lock on any processing related to the given
    * tree. If the lock is already held by the calling thread, this method 
    * will decrement the reference count. If the resulting reference count is
    * 0, the lock will be released. If the calling thread does not hold the
    * lock, this method will exit abnormally. This may be called recursively by
    * the same thread and must be called the same number of times as the
    * {@link #lockTree} method. Thus the calls to this unlock method must always
    * be balanced with calls to this method.
    * <p>
    * This locking has nothing to do with file system locks nor is there any
    * non-Java locking backing this lock. Rather, this lock is a cooperative
    * agreement between all callers (within the same JVM) that no use of the
    * given tree will be made except when the caller holds this exclusive lock
    * to the resource.
    *
    * @param    id
    *           Tree ID that is associated with the lock.
    *
    * @throws   IllegalMonitorStateException
    *           If the calling thread does not own the lock.
    * @throws   IllegalArgumentException
    *           If the ID is not a valid tree ID.
    */
   public void unlockTree(long id)
   throws IllegalMonitorStateException,
          IllegalArgumentException;

   /**
    * Access the next unassigned node ID for the specified tree. The
    * value is looked up and returned, just before returning the internal
    * cache is incremented and stored back for next use. Once this value is
    * returned, it is considered "used" and will not be returned again.
    *
    * @param    id
    *           The ID that specifies a tree.
    *
    * @return   The newly assigned node ID or {@link AstManager#INVALID_ID} if
    *           no such tree exists.
    *
    * @throws   AstException
    *           If too many AST nodes exist.
    */
   public long getNextNodeId(long id)
   throws AstException;

   /**
    * Resets the next unassigned AST node ID for the specified tree to the
    * first possible value (tree ID + 1).
    * <p>
    * <b>WARNING: there is no check on the current usage of node IDs for this
    * file, so if AST nodes exist already they will have IDs at or above this
    * value and duplicate IDs may (probably will) result!</b>
    *
    * @param    treeId
    *           The ID that specifies a valid tree.
    *
    * @throws   AstException
    *           If the file ID is invalid.
    */
   public void resetNextNodeId(long treeId)
   throws AstException;

   /**
    * Adds a mapping from the specified AST to a new ID. The next available
    * tree ID is assigned and this mapping is added to the registry. If the
    * tree name already exists, the existing tree ID is returned.
    *
    * @param    name
    *           The tree name. Often this will be the name of a file.
    *
    * @return   The assigned tree ID.
    *
    * @throws   AstException
    *           If too many ASTs exist in the registry or if the input
    *           name is <code>null</code>.
    */
   public long addTree(String name)
   throws AstException;

   /**
    * Removes all mapping data referencing the specified tree.
    *
    * @param    id
    *           The ID the tree to remove.
    *
    * @return   <code>true</code> if the tree existed in the registry (and 
    *           was removed), <code>false</code> if the tree didn't exist.
    */
   public boolean removeTree(long id)
   throws AstException;
   
   /**
    * Persistently store the current state of the AST manager plugin.
    *
    * @throws  AstException
    *          On any failure.
    */
   public void save()
   throws AstException;
   
   /**
    * Check if the AST specified by the given name exists already in the plugin's storage.
    * 
    * @param    name
    *           The name (including path) of the AST file to be checked.
    * 
    * @return   <code>true</code> if the specified AST is in the plugin's storage.
    */
   public boolean isExistingAst(String name);
}
