/*
** Module   : InMemoryRegistryPlugin.java
** Abstract : implements an in-memory XML storage option for an AstManagerPlugin
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20131023 Created first version.
*/

package com.goldencode.ast;

import java.util.*;

/**
 * Defines an in-memory
 */
public class InMemoryRegistryPlugin
extends BasePlugin
{
   /** In-memory storage of ASTs, by their name. */
   private final Map<String, Aast> asts = new HashMap<String, Aast>();

   /**
    * Load a named AST from the in-memory storage, by returning a copy of the {@link #asts}
    * entry after using the specified <code>name</code> as the key.
    *
    * @param    name
    *           The name of the tree. Often this is a file name.
    *
    * @return   The loaded AST.
    *
    * @throws   AstException
    *           On any error during the loading process.
    * @throws   IllegalArgumentException
    *           If the name is invalid or the specified resource doesn't exist.
    */
   @Override
   public Aast loadTree(String name)
   throws AstException,
          IllegalArgumentException
   {
      Aast ast = asts.get(name);
      
      Aast clone = ast.duplicate();
      clone.removeAnnotation("terse");
      
      if (ast.isAnnotation("terse") && (Boolean) clone.getAnnotation("terse"))
      {
         terseTree(clone);
      }
      
      return clone;
   }

   /**
    * Save the given AST to in-memory storage under the given name. This will save a copy of the
    * specified AST in the {@link #asts} map, usin the specified name as the key.
    *
    * @param    ast
    *           The tree to save.
    * @param    name
    *           The name of the tree. Often this is a file name.
    * @param    terse
    *           Do not store line and column data when the node data is saved.
    * 
    * @throws   AstException
    *           On any error during the saving process.
    * @throws   IllegalArgumentException
    *           If the name is invalid or the specified resource cannot be
    *           used as a target to store the AST.
    */
   @Override
   public void saveTree(Aast ast, String name, boolean terse)
   throws AstException,
          IllegalArgumentException
   {
      Aast clone = ast.duplicate();
      
      clone.putAnnotation("terse", terse);
      if (terse)
      {
         terseTree(clone);
      }
      
      asts.put(name, clone);
   }

   /**
    * A no-op for this in-memory storage.
    */
   @Override
   public void save()
   throws AstException
   {
      // no-op
   }
   
   /**
    * Removes this tree from the in-memory storage.
    *
    * @param    id
    *           The ID the tree to remove.
    *
    * @return   <code>true</code> if the tree existed in the registry (and 
    *           was removed), <code>false</code> if the tree didn't exist.
    *           
    * @see      BasePlugin#removeTree(long)
    */
   @Override
   public boolean removeTree(long id)
   throws AstException
   {
      String name = getTreeName(id);
      asts.remove(name);

      return super.removeTree(id);
   }

   /**
    * Check if the AST specified by the given name exists already in the plugin's storage.
    * 
    * @param    name
    *           The name (including path) of the AST file to be checked.
    * 
    * @return   <code>true</code> if the specified AST is in the plugin's storage.
    */
   public boolean isExistingAst(String name)
   {
      return asts.containsKey(name);
   }
   
   /**
    * Set the line and column information to 0 for the entire tree.
    * 
    * @param    ast
    *           The AST which needs the column/line information to be removed.
    */
   private void terseTree(Aast ast)
   {
      ast.setLine(0);
      ast.setColumn(0);
      
      Iterator<Aast> iter = ast.iterator();
      while (iter.hasNext())
      {
         ast = iter.next();
         ast.setLine(0);
         ast.setColumn(0);
      }
   }
}
